unit uclpv;
//////////////////////////////////////////////////////////////////////////////
//
// Basic implementation came from SWAG. Thanks a lot.
//
//////////////////////////////////////////////////////////////////////////////
//
// Author  : Thomas Kerkmann
// Date    : Jan. 1998
// Version : 1.0
// Release Notes:
//    This handy tool helped a lot on creating documentation for
//    some of our companies softwareproducts. With the autosave
//    option you just press Alt+Print, and the image of the current
//    window is saved as BMP, for later creation of helpfiles
//    or documents.
//
// Released to the Public Domain for experiments and education.
// Feel free to enhance, but give me a note if you can make it better.
// We all want to learn more. :-)
//
// e-mail   : thkerkmann@t-online.de
// homepage : http://home.t-online.de/home/thkerkmann/
//
//////////////////////////////////////////////////////////////////////////////
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, Menus;

type
  TMainFrmClpV = class(TForm)
    Image1           : TImage;
    StatusBar1       : TStatusBar;
    MainMenu1        : TMainMenu;
    Options1         : TMenuItem;
    Autosave1        : TMenuItem;
    setDirectory1    : TMenuItem;
    N1               : TMenuItem;
    File1            : TMenuItem;
    SaveAs1          : TMenuItem;
    N2               : TMenuItem;
    Exit1            : TMenuItem;
    SaveDialog1      : TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Autosave1Click(Sender: TObject);
    procedure ChangeOptions(Sender: TObject);
  private
    { Private-Deklarationen }
    FNextViewerHandle : THandle;
    FNr               : integer;
    FSaveName         : string;
    FSaveDir          : string;
    procedure ReadIniFile;
    procedure SaveIniFile;
    procedure WMDrawClipboard (var message : TMessage); message WM_DRAWCLIPBOARD;
    procedure WMChangeCBCHain (var message : TMessage); message WM_CHANGECBCHAIN;
    procedure ShowClip;
    procedure SaveClip;
  public
    { Public-Deklarationen }
  end;

var
  MainFrmClpV: TMainFrmClpV;

implementation
uses
   clipbrd, inifiles,
   
   uclpv2;
{$R *.DFM}

procedure TMainFrmClpV.ReadIniFile;
var
  aIni : tIniFile;
  fn   : string;
begin
  fn := ChangeFileExt (Application.ExeName,'.ini');
  aIni := tIniFile.Create (fn);
  with aIni do
   begin
     FSaveDir  := ReadString('Options','SaveDir','');
     FSaveName := ReadString('Options','FilePrefix','pic_');
     FNr       := ReadInteger('Options','Number',0);
   end;
  aIni.Free;
end;

procedure TMainFrmClpV.SaveIniFile;
var
  aIni : tIniFile;
  fn   : string;
begin
  fn := ChangeFileExt (Application.ExeName,'.ini');
  aIni := tIniFile.Create (fn);
  with aIni do
   begin
     WriteString ('Options','SaveDir',FSaveDir);
     WriteString ('Options','FilePrefix',FSaveName);
     WriteInteger('Options','Number',FNr);
   end;
  aIni.Free;
end;

procedure TMainFrmClpV.FormCreate(Sender: TObject);
begin
  // Hook the clipboard viewer chain
  // Should also check for a possible null return value, which indicates
  // that the function failed.
  FNextViewerHandle := SetClipboardViewer(Handle);

  ReadIniFile;
end;

procedure TMainFrmClpV.FormDestroy(Sender: TObject);
begin
  // Remove ourselves from the clipboard viewer chain.
  ChangeClipboardChain(Handle, FNextViewerHandle);
end;

procedure TMainFrmClpV.ShowClip;
begin
  SaveAs1.enabled := ClipBoard.HasFormat (CF_PICTURE);

  if SaveAs1.Enabled then
   begin
     Image1.Picture.Assign (ClipBoard);
     StatusBar1.SimpleText := Format ('Imageformat: %d x %d',
                                [Image1.Picture.Width,Image1.Picture.Height]);
   end
  else
     StatusBar1.SimpleText := 'No Bitmap-Format in Clipboard available';
  MessageBeep(-1);
end;

procedure TMainFrmClpV.SaveClip;
var
  fn : string;
begin
   inc (FNr);
   fn := FSaveDir+'\'+FSaveName+IntToStr(FNr)+'.bmp';
   while FileExists(fn) do
    begin
      inc (FNr);
      fn := FSaveDir+'\'+FSaveName+IntToStr(FNr)+'.bmp';
    end;
   Image1.Picture.SaveToFile (fn);
   StatusBar1.SimpleText := 'saved to '+fn;
end;

procedure TMainFrmClpV.WMDrawClipboard (var message : TMessage);
begin
  // Called whenever contents of the clipboard changes
  message.Result := SendMessage(WM_DRAWCLIPBOARD, FNextViewerHandle, 0, 0);

  ShowClip;
  if AutoSave1.Checked AND SaveAs1.enabled then
     SaveClip;
end;

procedure TMainFrmClpV.WMChangeCBCHain (var message : TMessage);
begin
  // Called when there is a change in the Clipboard viewer chain.
  if message.wParam = FNextViewerHandle then
   begin
     // the next viewer in the chain is being removed. Update our internal var.
     FNextViewerHandle := message.lParam;
     // Return 0 to indicate message was processed
     message.Result := 0;
   end
  else
   begin
     // Pass message on to next window in chain.
     message.Result := SendMessage(FNextViewerHandle, WM_CHANGECBCHAIN,
                       message.wParam, message.lParam);
   end;
end;

procedure TMainFrmClpV.Exit1Click(Sender: TObject);
begin
   SaveIniFile;
   Close;
end;

procedure TMainFrmClpV.SaveAs1Click(Sender: TObject);
var
  fn  : string;
  onr : integer;
begin
  SaveDialog1.InitialDir := FSaveDir;
  onr := FNr;
  inc (FNr);
  fn := FSaveDir+'\'+FSaveName+IntToStr(FNr)+'.bmp';
  while FileExists(fn) do
   begin
     inc (FNr);
     fn := FSaveDir+'\'+FSaveName+IntToStr(FNr)+'.bmp';
   end;
  SaveDialog1.FileName := FSaveName+IntToStr(FNr)+'.bmp';
  if SaveDialog1.Execute then
   begin
     Image1.Picture.SaveToFile (fn);
     StatusBar1.SimpleText := 'saved to '+fn;
   end
  else
     FNr := oNr;
end;

procedure TMainFrmClpV.Autosave1Click(Sender: TObject);
begin
  AutoSave1.Checked := NOT Autosave1.Checked;
end;

procedure TMainFrmClpV.ChangeOptions(Sender: TObject);
begin
  with dlgOptions do
   begin
     speStartNum.value := FNr;
     edFilePrefix.text := FSaveName;
     try
        dlbPath.Directory := FSaveDir;
     except
     end;
     if Image1.Stretch then
        rgImage.ItemIndex := 1   // stretch, no autosize
     else
        rgImage.ItemIndex := 0;  // no stretch, autosize
     if ShowModal=mrOK then
      begin
        FNr := speStartNum.value;
        FSaveName := edFilePrefix.text;
        FSaveDir  := dlbPath.Directory;
        Image1.AutoSize := rgImage.ItemIndex=0;
        Image1.Stretch  := rgImage.ItemIndex=1;
      end;
   end;
end;

end.
